import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.util.*;

/**
 * <b>Programm:</b> WaveThread<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert, Milan Altenburg<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> Class CalcThread, calculates Raster, holds array of Rasters
 */
public class CalcThread extends Thread{
  static final int MAX_QUEUE_SIZE = 10;
  int _x;     // current width
  int _y;     // current height
  float _t;   // seconds
  boolean _run;  // for main while() of thread
  
  // fifo "pointers" :)
  int _first;
  int _last;
  int _length;
  
  int[] _vals; // array for calculated values  
  MonoWave _mw;
  Color _color;
  DirectColorModel _dcm;
  WritableRaster[] _rasterqueue = new WritableRaster[MAX_QUEUE_SIZE];
  
  /**
   * CalcThread Constructor
   * sets time, MonoWave and Color
   * creates new RGB ColorModel
   */
  public CalcThread(MonoWave mw,Color c){
    _t  = 0;
    _mw = mw;
    _color = c;
  
    /* bits, bitmasks for r,g & b */
    _dcm = new DirectColorModel(24,0xff0000,0x00ff00,0x0000ff);
  }
  
  /**
   * sets new width and height
   * starts Thread, creates suitable Raster
   * resets queue values (length,first,last)
   */
  public synchronized void setXY(int x, int y){
    _x = x;
    _y = y;
    _run = true;
    _length=0;
    _vals = new int[3*_x*_y]; 
 
    // create suitable Rasters
    for (int i=0; i < MAX_QUEUE_SIZE; i++){
      _rasterqueue[i] = _dcm.createCompatibleWritableRaster(_x,_y);
    }
    
    // if first run call start()
    if(!this.isAlive()){
      this.start();
    }
  } 
  
  /**
   * @return size of queue
   */
  public int getQueueSize(){
    return(_length);
  }
  
  /**
   * @return the Raster, that is next
   */
  public WritableRaster getRaster() {    
    int tmp_first = _first; // hold value for return
    _first++;
    if(_first >= MAX_QUEUE_SIZE) _first = 0;
    _length--;
    if (_first == _last) _length=0;

    return(_rasterqueue[tmp_first]);
  }
  
  /**
   * control thread from outside, supposed but never used
   */
  public void die() {
    _run = false;
  }

  /**
   * calculates the next Raster, according to time _t
   * @param index, where to put Raster
   */
  public void calc(int idx){
    int base;
    
    //calc 1st row
    for (int i=0; i<_x; i++){
      base = 3 * i;
      float akt = _mw.getPsi((float) i,(float) _t );
      akt = (akt/_mw.getAmplitude())*_mw.getRatio(); //amplitude between -1.0 and 1.0
      
      float value = (akt+1)/2; // between 0.0 and 1.0
      
      _vals[base+0]= (int)(_color.getRed()  *value);
      _vals[base+1]= (int)(_color.getGreen()*value);
      _vals[base+2]= (int)(_color.getBlue() *value);
    }
    
    // fill other rows, copy
    for(int j=1; j<_y; j++) {
      for(int i=0;i<_x; i++) {
        base = 3*j*_x + 3 * i;
   
        _vals[base+0]= _vals[3*i+0];
        _vals[base+1]= _vals[3*i+1];
        _vals[base+2]= _vals[3*i+2];
      }
    }
    (_rasterqueue[idx]).setPixels(0,0,_x,_y,_vals);
  }

  /**
   * "main" of CalcThread
   * calls synchronized calc to fill queue
   */
  public void run(){
     while( _run ) {
       if(_length==0){
         _first=0;
	 _last=0;
	 _t += 0.2;
	 // avoid setXY() when calculating raster!
         synchronized(this){
           calc(0);
	   this.notify();
	 }
	 _length++;
       }else if(_length<MAX_QUEUE_SIZE-1 ){
         _last++;
	 if(_last >= MAX_QUEUE_SIZE) _last=0;
	 _t += 0.2;
	 synchronized(this){
	   calc(_last);
	   this.notify();
	 }
	 _length++;
       }else{
         yield();
       }
     }
  }
}
