import java.awt.*;
import java.awt.geom.*;

/**
 * <b>Programm:</b> WaveGradient<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert, Milan Altenburg<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> holds data for a monochromatic wave
 */
 
public class MonoWave{
  float _A;
  float _f;
  float _phi;
  float _lambda;
  float _scale;
  float _rotation;
  MonoWaveSlider _mwslider = null;
  GeneralPath _vpath;
  GeneralPath _hpath;

  /**
   * Constructor
   */
  public MonoWave(float A,float f,float lambda){
    //set up defaults:
    _mwslider = new MonoWaveSlider();
    _mwslider.getAmplitudeSlider().setFloatMinimum(0.0f);
    _mwslider.getAmplitudeSlider().setFloatMaximum(20.0f);
    _mwslider.getFrequencySlider().setFloatMinimum(0.0f);
    _mwslider.getFrequencySlider().setFloatMaximum(2.0f);
    _mwslider.getWaveLengthSlider().setFloatMinimum(0.0f);
    _mwslider.getWaveLengthSlider().setFloatMaximum(20.0f);
    _mwslider.getPhaseSlider().setFloatMinimum(-2.0f*(float)Math.PI);
    _mwslider.getPhaseSlider().setFloatMaximum( 2.0f*(float)Math.PI);
    
    setAmplitude(A);
    setFrequency(f);
    setWaveLength(lambda);
    setPhase(0.0f);
    setScale(50.0f);
    setRotationRad(0.0f);
    _vpath = new GeneralPath();
  }
  
  /**
   * sets the GeneralPath for vertical projection
   */
   
  public void setVerticalPath(GeneralPath gp){
    _vpath = gp;
  }
  
  /**
   * sets the GeneralPath for horizontal projection
   */
   
  public void setHorizontalPath(GeneralPath gp){
    _hpath = gp;
  }
  
  /**
   * returns GeneralPath for vertical projection
   */
   
  public GeneralPath getVerticalPath(){
    return _vpath;
  }
  
  /**
   * returns GeneralPath for horizontal projection
   */
   
  public GeneralPath getHorizontalPath(){
    return _hpath;
  }
  /**
   * Return MonoWaveSliderobject
   */
  public MonoWaveSlider getMonoWaveSlider(){
    return _mwslider;
  }


  /**
   * sets amplitude
   */
  public void setAmplitude(float A) {
    _A = A;
    _mwslider.getAmplitudeSlider().setFloatValue(A);
  }
  
  /**
   * sets wavelength
   */
  public void setWaveLength(float lambda) {
    _lambda = lambda;
    _mwslider.getWaveLengthSlider().setFloatValue(lambda);
  }
  
  /**
   * sets phaseshift
   */
  public void setPhase(float phi) {
    _phi = phi;
    _mwslider.getPhaseSlider().setFloatValue(phi);
  }

  /**
   * sets frequency
   */
  public void setFrequency(float f) {
    _f = f;
    _mwslider.getFrequencySlider().setFloatValue(f);
  }
  
  /**
   * sets scale
   */
  public void setScale(float scale) {
    _scale = scale;
  }

  /**
   * sets rotation in Degrees
   */
  public void setRotationDeg(float rotation) {
    rotation = rotation % 360;
    _rotation = (rotation*2*(float)Math.PI)/360;
  }

  /**
   * sets rotation in Radians
   */
  public void setRotationRad(float rotation) {
    rotation = rotation % (2*(float)Math.PI);
    _rotation = rotation;
  }


  /**
   * gets amplitude
   */
  public float getAmplitude() {
    return(_A);
  }
  
  /**
   * gets wavelength
   */
  public float getWaveLength() {
    return (_lambda);
  }
  
  /**
   * gets phaseshift
   */
  public float getPhase() {
    return (_phi);
  }
  
  /**
   * gets frequency
   */
  public float getFrequency() {
    return(_f);
  }

  /**
   * gets circlefrequency (omega)
   */
  public float getCircleFrequency(){
    return (2*(float)Math.PI*_f);
  }
 
  /**
   * gets Wavekey (k)
   */
  public float getWaveKey(){
    return (2*(float)Math.PI/_lambda);
  }
  
  /**
   * gets Psi
   */
  public float getPsi(float x,float t) {
    x = x/_scale;
    t = t/_scale;
    
    return(_A*(float)Math.sin( getWaveKey()*x - getCircleFrequency()*t + getPhase()));
  }
  
  /**
   * gets scale
   */
  public float getScale() {
    return(_scale); 
  }

  /**
   * gets rotation in degrees
   */
  public float getRotationDeg() {
    return( (360*_rotation) / (2*(float)Math.PI) ); 
  }

  /**
   * gets rotation in radians
   */
  public float getRotationRad() {
    return(_rotation); 
  }

  
  /**
   * gets ratio between SliderMaximum and Amplitude
   */
  public float getRatio() {
    return(_A/_mwslider.getAmplitudeSlider().getFloatMaximum()); 
  }

}
